/*
   This version of the client sends multiple lines of output
   to the server. After sending all of its requests to the
   server, this client closes its output stream to the server
   (which causes the server to see an end-of-file condition)
   and then this client expects one response back from the
   server.
*/
import java.net.*;
import java.io.*;
import java.util.Scanner;

public class EchoClient_v3
{
   public static final int SERVER_PORT = 5000; // Should be above 1023.

   public static void main (String[] args)
   {
      Socket          socket = null;
      BufferedReader  in = null;
      PrintWriter     out = null;

      final String hostName;
      if (args.length > 0)
      {
         hostName = args[0];
      }
      else
      {
         hostName = "localhost";
      }

      final int portNumber;
      if (args.length > 1)
      {
         portNumber = Integer.parseInt(args[1]);
      }
      else
      {
         portNumber = SERVER_PORT;
      }

      // Get this client's process id number (PID). This helps
      // to identify the client in the server's transcrip.
      final ProcessHandle handle = ProcessHandle.current();
      final long pid = handle.pid();
      System.out.println("CLIENT: Process ID number (PID): " + pid );

      // Make a connection to the server
      try
      {
         System.out.println("CLIENT: connecting to server: " + hostName + " on port " + portNumber );
         socket = new Socket(InetAddress.getByName(hostName), portNumber);

         in = new BufferedReader(
                  new InputStreamReader(
                       socket.getInputStream()));
         out = new PrintWriter(socket.getOutputStream());
      }
      catch (IOException e)
      {
         System.out.println("CLIENT: Cannot connect to server.");
         //System.out.println( e );
         e.printStackTrace();
         System.exit(-1);
      }

      // Send the server multiple requests.
      final Scanner stdin = new Scanner(System.in);
      System.out.println("Type a message for the server (^z or ^d to end):");
      while ( stdin.hasNextLine() )
      {
         final String message = stdin.nextLine();
         out.println(message);
         out.flush();
         System.out.println("Type a message for the server (^z or ^d to end):");
      }

      // Tell the server that this client is done.
      // We cannot close the socket's out stream. That shuts down the socket.
      // Instead we tell the socket to close its output stream.
      try
      {
         //out.close(); // Wrong!
         socket.shutdownOutput();
      }
      catch (IOException e)
      {
         System.out.println("CLIENT: Cannot shut down output stream to server.");
         System.out.println( e );
      }

      //Wait for a single response from the server.
      try
      {
         final String response = in.readLine();
         System.out.println("CLIENT: Server response is: \"" + response + "\"");
      }
      catch (IOException e)
      {
         System.out.println("CLIENT: Cannot receive response from server.");
         System.out.println( e );
      }

      // Disconnect from the server.
      try
      {
         socket.close();
         System.out.println("CLIENT: Closed socket.");
      }
      catch (IOException e)
      {
         System.out.println("CLIENT: Cannot disconnect from server.");
         System.out.println( e );
      }
   }
}
